// $Id: DRAWxtlViewUI.h 502 2006-08-19 22:50:29Z Larry $
//
// DRAWxtl V5.3 main window header
// initial version generated by Fast Light User Interface Designer (fluid) version 1.0104
// Coded using the FLTK 1.1.6 widget set
//
//     Larry W. Finger, Martin Kroeker and Brian Toby
//

#ifndef DRAWxtlViewUI_h
#define DRAWxtlViewUI_h
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Menu_.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Progress.H>
#include <FL/x.H>
#include "drawxtl.h"
#include "CrystalView.h"
#include "Draw_Fl_Input.h"

#include "DRAWxtl_proto.h"

enum {
	LIST1 = 1<<0,
	LIST2 = 1<<1,
	LIST3 = 1<<2,
	HELP  = 1<<3,
	HELP1 = 1<<4,
	TEXTWIN = 1<<5,
	SPHERE = 1<<6,
	ELLIPSOID = 1<<7,
	LONEPAIR = 1<<8,
	TEXT1 = 1<<9,
};

class DRAWxtlViewUI {
public:
  void show(int , char **);
  DRAWxtlViewUI();                  // constructor

// pointers to widgets

  Fl_Multiline_Output * Cursor_pos; 
  Fl_Output * CurDirBuf;
  Fl_Output * CurStrBuf;
  Fl_Text_Buffer * CutoutColorBuf;
  Fl_Double_Window *mainWindow;
  Fl_Check_Button *Use_Clipping;
  Flu_Spinner *X_Min;
  Flu_Spinner *Y_Min;
  Flu_Spinner *Z_Min;
  Flu_Spinner *X_Max;
  Flu_Spinner *Y_Max;
  Flu_Spinner *Z_Max;
  Flu_Spinner *X_Min_clip;
  Flu_Spinner *Y_Min_clip;
  Flu_Spinner *Z_Min_clip;
  Flu_Spinner *X_Max_clip;
  Flu_Spinner *Y_Max_clip;
  Flu_Spinner *Z_Max_clip;
  Draw_Fl_Input *Origin_X;
  Draw_Fl_Input *Origin_Y;
  Draw_Fl_Input *Origin_Z;
  Draw_Fl_Input *X_Rot;
  Draw_Fl_Input *Y_Rot;
  Draw_Fl_Input *Z_Rot;
  Fl_Output *CurFile;
  Fl_Output *CurDir;
  Fl_Check_Button *Generate_VRML1;
  Fl_Output * Origin1_Msg;
  Fl_Check_Button *Orthographic_View;
  Fl_Check_Button *Show_Vector_Triple;
  static Fl_Menu_Item drawxtl_menu[];
  Flu_Combo_List * Frame_No;
  Fl_Group *ViewGroup;
  Fl_Box *cframe;
  CrystalView *crystal;

// variables used in drawing

  GLuint crystalDL;
  int destroy;               // window destructor flag variable
  int frame_no;              // number of frame being processed
  int max_frame;             // maximum number of frames
  char col_cell[40];         /* color and properties of framebars */
  int Str_File_Changed;      // set true (==1) if contents of str file changed
  float X_Boxlim;            // X box limit read from command line
  float Y_Boxlim;            // Y box limit read from command line
  float Z_Boxlim;            // Z box limit read from command line
  float X_Origin;            // X origin read from command line
  float Y_Origin;            // Y origin read from command line
  float Z_Origin;            // Z origin read from command line
  float lookat_v1[3], lookat_v2[3]; // "lookat" vectors "
  float label_scale;         // relative scaling of text labels
  int noshadow;              // if true, no shadows in POV
  float ambient;             // POV lighting options
  float diffuse;             //
  float specular;            //
  float roughness;           //
  int Stereo;                // TRUE for stereo views
  int cross_eyed;            // TRUE for "cross-eyed" viewing
  float stereo_base;         // separation wanted
  float clip_lim[MAX_FRAME][6]; /* limits of clip box in cell coords */
  float cryst_lim[MAX_FRAME][6];/* limits of view box in cell coords */
  float Trans[3];            // translation of virtual trackball
#if !defined (WIN32) && !defined (__APPLE__)
  Pixmap icon;               // icon used only for Linux/Unix 
#endif

// symmetry variables
  
  char spg[3][4];            // space group code
  int ss[24][3][3];          // rotation operators for symmetry
  float rss[24][3][3];       // unitary rotation operators for symmetry
  float lat_pos[4][3];       // coordinates of lattice points
  float ts[24][3];           // translation operators
  int sym_op_no[192];        // saved symmetry operation number
  float cell_xyz[192][3];    // storage for generated equiv. positions
  int sys;                   // crystal system indicator
  int nbr;                   // Bravais lattice indicator
  int acentric;              // true if non-centric space group
  int ng;                    // number of rotation operations to specify group
  int nlat;                  // number of lattice positions
  float xyzoff[3];           // offset between origin and center of symmetry
  int xyzoff_read;           // true if xyzoff line read
  int origin1_flag;          // true if space group has alternate origin setting
  float origin_offset[3];    // offset vector between settings

// lattice constant variables
  
  float lat_con[6];          // lattice constants
  float rec_lat_con[6];      // reciprocal lengths and cosines of angles
  double b_mat[3][3];        // metric tensor
  float ginv[3][3];          // direct space metric tensor

// lattice parameters of cutout box 

  float slab_con[6];         // lattice constants
  float slab_off[3];         // offset
  float slab_rot[3];         // rotation angles
  
// variables for atom lists

  float atom_xyz[MAX_ATOM][3];// atomic coordinates
  char atom_l[MAX_ATOM][5];  // atom labels
  int atom_n[MAX_ATOM];      // input atom number
  int atom_fn[MAX_ATOM];     // frame number for atom
  int sv_atom_n[MAX_ATOM];   // saved input atom number
  int *atom_no;              // master list atom and symmetry number
  int *orig_atom_no;         // saved atom number
  int TF_status[MAX_ATOM];   // Temperature factor status (-1 no Uij, 0 sphere, 1 ellipsoid)

// atom list variables for modulated structures
  int modulated;	     // flag for modulated or conventional/average structure
  int no_mod_vectors;        // number of atom_site_fourier_wave_vectors
  int no_site_displace;      // number of atom_site_displace_fourier terms
  int no_site_U_terms;       // number of atom_site_U_fourier_terms
  int no_site_occ;           // number of atom_site_occ_fourier terms
  int no_cell_vec;           // number of cell_wave_vectors
  float cell_vec[3][3];      // cell_wave_vectors
  int no_subsys;             // number of subsystems
  float subsys_ref_volume;   // reference reciprocal volume
  float subsys_vol[10];      // reciprocal volume for each of the subsystems
  float subsys_fact[10][3][3]; // place to save relationship between main and subsystem cells
  int   vector_mult[MAX_MODULATION][3]; // coefficients ijk for modvector = i*cell_vec_1 + j*cell_vec_2 ..
  float modvector[MAX_MODULATION][3];   // atom Fourier modulation vectors
  float atom_modpar[3*MAX_ATOM*MAX_MODULATION][2]; // cosine and sine multiplier for an item
  float atom_mod_sawtooth[MAX_ATOM*MAX_MODULATION][5]; // amplitudes and interval for sawtooth modulation
  int   atom_modpar_atom[3*MAX_ATOM*MAX_MODULATION]; // atom number for atom_modpar
  int   atom_modpar_axis[3*MAX_ATOM*MAX_MODULATION]; // axis number (0-2) for atom_modpar
  int   atom_modpar_id[3*MAX_ATOM*MAX_MODULATION]; // entry in vector_mult for this term (1 - no_mod_vectors)
  float atom_occpar[MAX_ATOM*MAX_MODULATION][2]; // cosine and sine multiplier of occ fourier series
  int   atom_occpar_atom[MAX_ATOM*MAX_MODULATION]; // atom number for atom_occpar
  int   atom_occpar_id[MAX_ATOM*MAX_MODULATION]; // axis number (0-2) for atom_occpar
  float atom_occ_crenel[MAX_ATOM*MAX_MODULATION][2];   // crenel lower and upper limit for occupancy
  int   atom_ismod[MAX_ATOM];                   // flag for modulated position
  int   occ_ismod[MAX_ATOM];                   	// flag for modulated occupancy
  float occupancy[MAX_ATOM];                    // average occupancy
  float min_occ[MAX_ATOM];                      // minimum occupancy required for drawing the atom
  int   ellips_ismod[MAX_EL];                   // flag for modulated Uij
  float ellips_modpar[3*MAX_EL*MAX_MODULATION][2];  //wave vector id, cosine and sine multip
  int   ellips_modpar_id[3*MAX_EL*MAX_MODULATION];  // entry for this term (1 - no_mod_vectors)
  int   ellips_modpar_term[3*MAX_EL*MAX_MODULATION]; // identity of this term (1-6);
  int   ellips_modpar_atom[MAX_ATOM*MAX_MODULATION]; // atom number for this item
  float phaseshift[3];                         // modulation phase offset t for each modulation
  float * vert_occ;                            // occupancy at individual sites (for radius scaling)
  float ts_m[24][3];                           // translational component of superspace symmetry
  int ss_m[24][3][3];                          // rotational component of superspace symmetry

  
// cone parameters
  
  int ncone;                  // number of different types of lonepair cones
  char cone_l1[MAX_CONE][5];  // label for atom end of cone
  int numlonepairs[MAX_CONE]; // number of lone pairs to draw at atom
  float cone_height[MAX_CONE];// height of cone
  float cone_min[MAX_CONE];   // cone tip radius
  float cone_max[MAX_CONE];   // cone base radius
  char col_cone[MAX_CONE][40];// color and properties of cones for POV
  int cone_fn[MAX_CONE];      // frame number for the cone commands

// magnetic arrow parameters

  int nmag;                       // number of magnetic moments
  int arrow_fn[MAX_ARROW];        // saved frame number
  char col_arrow[MAX_ARROW][40];// arrow color
  float arrow_length[MAX_ARROW];  // length of mag. arrow
  float arrow_diam[MAX_ARROW];    // diameter of magnetic arrow
  float mag_matrix[3][3];         // transformation between magnetic and nuclear cell
  float mag_xp[MAX_ARROW][3];     // position of arrow in real-space cell
  float mag_xc[MAX_ARROW][3];     // components of arrow wrt XYZ

// polyhedra parameters

  int npoly;                  // number of different types of polyhedra
  int poly_fn[MAX_POLY];      // frame number for polyhedra commands
  float polylimit;            // should be close to zero to allow only regular polygonal faces
  char poly_l[MAX_POLY][5];   // polyhedral center label
  char poly_t[MAX_POLY][5];   // polyhedral vertex label
  float poly_size[MAX_POLY];  // size of polyhedron
  float poly_min[MAX_POLY];   // lower limit for shell-like polyhedron
  char poly_col[MAX_POLY][40];// polyhedron colors for POV
  float poly_rad_edge[MAX_POLY]; // individual edge radius for polyhedron
  char poly_col_edge[MAX_POLY][40]; // color of individual poly edges for POV
  char col_edge[40];          // default color for edge
  float rad_edge;             // default radius for edge

// plane parameters

  int nplane;                      // number of different types of planes
  int plane_fn[MAX_PLANE];         // frame number for plane commands
  char plane_col[MAX_PLANE][40];   // plane colors for POV 
  char plane_l[MAX_PLANE][5];      // plane center label
  float plane_size[MAX_PLANE];     // size of plane coordination

// bond parameters

  int nbond;                       // number of different types of bonds
  int bond_fn[MAX_BOND];           // frame number for bond commands
  char bond_l1[MAX_BOND][5];       // label for one end of bond
  char bond_l2[MAX_BOND][5];       // label for other end
  float bond_size[MAX_BOND];       // bond diameter
  float bond_min[MAX_BOND];        // minimum bond distance
  float bond_max[MAX_BOND];        // maximum bond distance
  char col_bond[MAX_BOND][40];     // color and properties of bonds for POV
  int bond_style[MAX_BOND];	   // solid bond or dash
  float Bond_Mult;                 // Multiplier for Bond Size
  float BndMult;                   // Bond multiplier initialization
  float mol_d;                     // intramolecular distance for molecular and polyhedral completion

// sphere parameters

  int sphere_fn[MAX_SPHERE];         // frame number for sphere commands
  char sphere_l[MAX_SPHERE][5];      // sphere label
  int  sphere_n[MAX_SPHERE];         // atom number - -1 for old behavior
  float sphere_size[MAX_SPHERE];     // sphere size
  char sphere_col[MAX_SPHERE][40];   // sphere colors
  float Sphere_Mult;                 // Multiplier for Sphere Size
  float SpMult;                      // initialization for Sphere_Mult
  int nsphere;                       // number of different types of spheres


// background parameters

  char col_bg[40];             // Background color in POV file
  float glback[3];             // Background RGB values for openGL

// ellipsoid parameters

  int  auto_ellipse;             // true if ellipsoids should be automatically generated
  char ellips_col[MAX_EL][40];   // ellipsoid colors for POV
  float ellips_EV[MAX_EL][3][3]; // Eigenvectors
  float ellips_RMS[MAX_EL][3];   // RMS amplitudes
  char ellips_l[MAX_EL][5];      // label for anisotropic atom
  int ellips_n[MAX_EL];          // atom number
  int save_el_number[MAX_EL];    // save original atom number
  int ell_type[MAX_EL];          // form of ellipsoid input
  float ellips[MAX_EL][6];       // thermal ellipsoid coefficients
  float Phong_Value;             // reflectivity value for Phong highlighting
  float Phong_Size;              // Phong highlighting size
  char Cutout_color[40];         // color of cutout section
  char Ellipaxis_color[40];      /* color of principal axes */
  int n_ellips;                    // no. of aniso. atoms in asymmetric unit
  int do_ellipsoids;               // non-zero if ellipsoids to be drawn
  int El_Cutout;                   // True if ellipsoids to have an octant removed
  float Ellipsoid_Prob;            /* Fraction of electron dens enclosed by ellipsoid */
  float Ellipsoid_Scale;           /* Scaling needed to get probability */
  float Ellipaxis_width;           /* linewidth of principal axes */

// text label parameters

  int nlabel;                    // number of textual labels
  int label_fn[MAX_LABELS];      // frame number
  float label_x[MAX_LABELS][3];  // position of textual labels
  char label_label[MAX_LABELS][64]; // label text
  float saved_x_label[2][4][3];  // place to save coordinates of each type of label
  int labels_inited;             // set to true when labels initialized
  int triple[4];                 // label numbers of the parts of the triple vector
  int autolabel;		 // set to true when import adds atom labels

// least squares plane parameters

  int nbplane;				     // number of least squares planes
  char bplane_t[MAX_BPLANE][10][5];          // atom labels in plane list
  int bplane_n[MAX_BPLANE][10];              // atom numbers in plane list
  int nbatoms[MAX_BPLANE];                   // number of items in list
  float bplane_d1[MAX_BPLANE];               // width of plane segment to draw
  float bplane_d2[MAX_BPLANE];		     // second dimension
  char bplane_col[MAX_BPLANE][40];           // color
  
// miscellaneous variables

  double Old_Xrot;               // Used to determine if str file changed
  double Old_Yrot;               // Used to determine if str file changed
  double Old_Zrot;               // Used to determine if str file changed
  float cur_step;
  char * table;                  // used to store miscellaneous character data

// molecule variables

  char molecule_atom_name[MAX_FRAME][5];
  int molecule_atom_no[MAX_FRAME];
  float molecule_distance[MAX_FRAME];

// file variables

  FILE *fpin, *flout;        
  FILE *fpoutv;
  FILE *fpoutp;
  FILE *fcns;
  char Cur_Console[256];     // file strings
  char Cur_Dir[256];
  char Cur_File[256];
  char Cur_Temp[256];
  char Cur_Listing[256];
  char Cur_Root[256];
  char DRAWxtl_Path[256];
  char EditName[256];
  char FileViewName[256];
  char POV_Include[256];
  char POV_Options[256];
  char POV_Path[256];
  char DefaultFinish[256];
  char VRML_Path[256];
  char LoadOnStartup[10];
  char ProgramPath[256];
};

#endif
